(*	GEDitCOM II Download Interface Format
	24 Jun 2010, by John A. Nairn
	
	Download list of available formats at
	
	     http://www.geditcom.com/extensions/FileList.txt
	
	and let user pick one to download.
	
	Once selected, download zip file to target folder and expand it
*)

property scriptName : "Download Extension"
property appName : "GEDitCOM II"

-- read the list of available extensions
-- Each extension has two lines. First is relative path to zip file
--   which is just name for plug in and Styles/name ro formats
-- Second line is text to put in displayed list
set intFiles to (do shell script "curl http://www.geditcom.com/extensions/FileList.txt")

-- quit if fails - the first line is a comment beginning in "#"
if character 1 of intFiles is not "#" then
	tell application "GEDitCOM II"
		activate
		user option title 
			"The list of available extensions could not be read" message 
			"Check your Internet connection and try again" buttons {"OK"}
	end tell
	return
end if

-- break list into zip file names and display name for the list
set fileNames to {}
set fileDescrip to {}
repeat with i from 2 to (number of paragraphs in intFiles) by 2
	set end of fileNames to paragraph i of intFiles
	set end of fileDescrip to paragraph (i + 1) of intFiles
end repeat

-- let user pick one of the file
tell application "GEDitCOM II"
	activate
	if last word of appName is "II" then
		set newForm to user choice list items fileDescrip 
			prompt 
			"Select extension to download from following cateogries:" buttons {"To System Extensions", "To User Extensions", "Cancel"} title scriptName
	else
		set newForm to user choice list items fileDescrip 
			prompt 
			"Select extension to download from following cateogries:" buttons {"Install", "Cancel"} title scriptName
	end if
end tell

-- exit if canceled
set btn to item 1 of newForm
if btn is "Cancel" then
	return
else if btn is "To System Extensions" or btn is "Install" then
	set toDomain to "System"
else
	set toDomain to "User"
end if

-- find which one was selected and get file URL
set downIndex to item 1 of item 3 of newForm

-- relative path on server to the file and get full URL
set zipFile to item downIndex of fileNames

-- count length of zipFile
set nc to number of characters in zipFile

-- get URL of src file
set srcPath to "http://www.geditcom.com/extensions/" & zipFile

-- flatten zipFile (remove dwnlds/)
set zipFile to (characters 8 thru nc of zipFile) as string

-- a plug in or an extension?
if (offset of ".gfrmt." in zipFile) > 0 then
	-- get flat file name (e.g. "Default Format.gfrmt")
	-- A browser style gets file name for text to user
	set expandSave to item downIndex of fileDescrip
	
	-- remove header ending in : if there
	set coff to offset of ":" in expandSave
	if coff > 0 then
		set nc to number of characters in expandSave
		set expandSave to (characters (coff + 1) thru nc of expandSave) as string
	end if
	
	-- remove leading space
	set nb to 1
	repeat until character nb of expandSave is not " "
		set nb to nb + 1
	end repeat
	set nc to number of characters in expandSave
	set expandSave to (characters nb thru nc of expandSave) as string
	set expandSave to expandSave & ".gfrmt"
else
	-- flat file name once expanded
	set nc to number of characters in zipFile
	set expandSave to (characters 1 thru (nc - 4) of zipFile) as string
end if

-- get paths to main app support folder
set appSupport to path to application support from user domain as string
set appSupport to appSupport & "GEDitCOM II:"

-- the zip file on geditcom.com is transferred to root
-- level of the App support folder (using just its name)
-- this is temporary storage
set destPath to appSupport & zipFile

-- expandPath is to save result of expanding the downloaded zip file
--  and it will be at root level of the app support folder
set expandPath to appSupport & expandSave

-- unzipFolder is path to folder to get expanded archive
-- (i.e., the app support folder root level)
set unzipFolder to appSupport

-- delete prior copies if there
try
	tell application "Finder"
		if exists file destPath then
			delete file destPath
		end if
		if exists file expandPath then
			delete file expandPath
		end if
	end tell
on error
	tell application "GEDitCOM II"
		user option title 
			"A previous copy of the extension could not be replaced" message 
			"You may have a file permission problem" buttons {"OK"}
	end tell
	return
end try

-- download zip file
try
	set myFile to POSIX path of file destPath
	do shell script "curl -L " & srcPath & " -o '" & myFile & "'"
on error
	tell application "GEDitCOM II"
		user option title 
			"The extension file could not be downloaded" message 
			"You can try again later or manually download and expanding from geditcom.com." buttons {"OK"}
	end tell
	return
end try

-- expand the zip file
-- zipPath is path to file that was just downloaded
-- outPath is path to folder to get the unzipped file or folder
set zipPath to POSIX path of file destPath
set outPath to POSIX path of file unzipFolder
set zipcmd to "ditto -x -k '" & zipPath & "' '" & outPath & "'"
try
	do shell script zipcmd
on error
	tell application "GEDitCOM II"
		user option title 
			"The downloaded extension could not be expanded" message 
			"You can try downloading and/or expanding manually." buttons {"OK"}
	end tell
	return
end try

-- delete the downloaded zip file
tell application "Finder"
	delete file destPath
end tell

-- now have the extension file at root level of GEDitCOM II
-- app support folder. Need to install it.

-- install the extension according to btn text
tell application "GEDitCOM II"
	set outPath to POSIX path of file expandPath
	install extension outPath domain toDomain
end tell

-- delete expanded extension
tell application "Finder"
	delete file expandPath
end tell

-- refresh scripts menu
tell application "GEDitCOM II"
	refresh scripts
end tell


